#!/usr/bin/env python3
"""
Command-line interface for wl_version_manager
"""

import sys
import argparse
from .version_manager import VersionManager


def main():
    # Get version for --version flag
    import os
    from pathlib import Path
    version = "unknown"
    try:
        # Try to get version from package
        from importlib.metadata import version as get_version
        version = get_version('wl_version_manager')
    except:
        # Fallback to reading VERSION file in package directory
        version_file = Path(__file__).parent.parent / 'VERSION'
        if version_file.exists():
            version = version_file.read_text().strip()
    
    parser = argparse.ArgumentParser(
        description="Manage package versions - by default only updates VERSION file",
        epilog="By default, only the VERSION file is updated. Use --update-package-json or "
               "--update-setup to update other files, or create a .versionrc config file."
    )
    parser.add_argument('--version', action='version', version=f'wl_version_manager {version}')
    parser.add_argument("--version-file", default="VERSION",
                       help="Version file path (default: VERSION)")
    parser.add_argument("--setup-file", default="setup.py",
                       help="Setup file path (default: setup.py)")

    # Global flags for file control
    parser.add_argument("--update-setup", action="store_true",
                       help="Also update setup.py")
    parser.add_argument("--skip-setup", action="store_true",
                       help="Skip updating setup.py (even if in config)")
    parser.add_argument("--update-package-json", action="store_true",
                       help="Also update package.json files")
    parser.add_argument("--skip-package-json", action="store_true",
                       help="Skip updating package.json files (even if in config)")
    parser.add_argument("--package-file", action="append",
                       help="Specific package.json file to update (can be used multiple times)")

    subparsers = parser.add_subparsers(dest="command", help="Available commands")

    # Current version command
    subparsers.add_parser("current", help="Show current version")

    # Bump commands
    subparsers.add_parser("patch", help="Bump patch version")
    subparsers.add_parser("minor", help="Bump minor version")
    subparsers.add_parser("major", help="Bump major version")

    # Set version command
    set_parser = subparsers.add_parser("set", help="Set specific version")
    set_parser.add_argument("version", help="Version to set (e.g., 1.2.3)")

    # Init command
    init_parser = subparsers.add_parser("init", help="Initialize VERSION file")
    init_parser.add_argument("--initial-version", default="0.1.0",
                           help="Initial version (default: 0.1.0)")

    # Config command
    subparsers.add_parser("config", help="Create .versionrc config file")

    args = parser.parse_args()

    if not args.command:
        parser.print_help()
        return 1

    vm = VersionManager(args.version_file, args.setup_file)

    # Build update_files dict from flags
    update_files = {}
    # Check if attributes exist before accessing
    if hasattr(args, 'update_setup') and args.update_setup:
        update_files['setup.py'] = True
    if hasattr(args, 'skip_setup') and args.skip_setup:
        update_files['setup.py'] = False
    if hasattr(args, 'update_package_json') and args.update_package_json:
        update_files['package.json'] = True
    if hasattr(args, 'skip_package_json') and args.skip_package_json:
        update_files['package.json'] = False

    # Override _update_version if we have custom flags
    if update_files or (hasattr(args, 'package_file') and args.package_file):
        original_update = vm._update_version
        def custom_update(version):
            vm.write_version(version)

            # Check setup.py updates
            setup_enabled = update_files.get('setup.py', vm.config.get('files', {}).get('setup.py', False))
            if not (hasattr(args, 'skip_setup') and args.skip_setup) and setup_enabled:
                vm.update_setup_py(version)

            # Check package.json updates
            package_enabled = update_files.get('package.json', vm.config.get('files', {}).get('package.json', False))
            if not (hasattr(args, 'skip_package_json') and args.skip_package_json):
                if hasattr(args, 'package_file') and args.package_file:
                    # Use specific files
                    vm.update_package_json(version, args.package_file)
                elif package_enabled:
                    # Use config or auto-discover
                    vm.update_package_json(version)

        vm._update_version = custom_update

    try:
        if args.command == "init":
            vm.write_version(args.initial_version)
            if hasattr(args, 'update_setup') and args.update_setup:
                vm.update_setup_py(args.initial_version)
            if hasattr(args, 'update_package_json') and args.update_package_json:
                vm.update_package_json(args.initial_version, args.package_file if hasattr(args, 'package_file') else None)
            print(f"Initialized version to {args.initial_version}")

        elif args.command == "current":
            print(vm.read_version())

        elif args.command == "config":
            vm.create_config()

        elif args.command == "patch":
            new_version = vm.bump_patch()
            print(f"Bumped to {new_version}")

        elif args.command == "minor":
            new_version = vm.bump_minor()
            print(f"Bumped to {new_version}")

        elif args.command == "major":
            new_version = vm.bump_major()
            print(f"Bumped to {new_version}")

        elif args.command == "set":
            new_version = vm.set_version(args.version)
            print(f"Set version to {new_version}")

    except Exception as e:
        print(f"Error: {e}", file=sys.stderr)
        return 1

    return 0


if __name__ == "__main__":
    sys.exit(main())