import re
import os
import sys
import json
from pathlib import Path


class VersionManager:
    def __init__(self, version_file="VERSION", setup_file="setup.py"):
        self.version_file = Path(version_file)
        self.setup_file = Path(setup_file)
        self.config = self._load_config()

    def _load_config(self):
        """Load configuration from .versionrc if it exists"""
        config_file = Path(".versionrc")
        if config_file.exists():
            try:
                with open(config_file, 'r') as f:
                    return json.load(f)
            except Exception as e:
                print(f"Warning: Could not load .versionrc: {e}")
        
        # Default config
        return {
            "files": {
                "version": True,
                "setup.py": True,
                "package.json": False
            },
            "package_json_paths": [],
            "auto_discover": True
        }

    def read_version(self):
        """Read current version from VERSION file"""
        if not self.version_file.exists():
            raise FileNotFoundError(f"Version file {self.version_file} not found")
        return self.version_file.read_text().strip()

    def write_version(self, version):
        """Write version to VERSION file"""
        self.version_file.write_text(version + '\n')

    def update_setup_py(self, version):
        """Update version in setup.py"""
        # First check if we should even update setup.py
        if not self.config.get('files', {}).get('setup.py', False):
            return  # Not configured to update, silently skip
            
        if not self.setup_file.exists():
            print(f"Warning: {self.setup_file} not found, skipping setup.py update")
            return

        content = self.setup_file.read_text()

        # Pattern to match version= line
        pattern = r'version\s*=\s*["\'][^"\']*["\']'
        replacement = f'version="{version}"'

        if re.search(pattern, content):
            new_content = re.sub(pattern, replacement, content)
            self.setup_file.write_text(new_content)
        else:
            print(f"Warning: Could not find version= line in {self.setup_file}")

    def find_package_json_files(self, root_dir="."):
        """Find all package.json files in the project"""
        package_files = []
        for root, dirs, files in os.walk(root_dir):
            # Skip node_modules directories
            if 'node_modules' in dirs:
                dirs.remove('node_modules')
            if '.git' in dirs:
                dirs.remove('.git')
            
            if 'package.json' in files:
                package_files.append(Path(root) / 'package.json')
        return package_files

    def update_package_json(self, version, package_files=None):
        """Update version in package.json file(s)"""
        if package_files is None:
            # Use config paths or auto-discover
            if self.config.get('package_json_paths'):
                package_files = [Path(p) for p in self.config['package_json_paths']]
            elif self.config.get('auto_discover', True):
                package_files = self.find_package_json_files()
            else:
                return
        
        if not package_files:
            return
        
        for pf in package_files:
            try:
                with open(pf, 'r') as f:
                    data = json.load(f)
                
                old_version = data.get('version', 'unset')
                data['version'] = version
                
                with open(pf, 'w') as f:
                    json.dump(data, f, indent=2)
                    f.write('\n')  # Add trailing newline
                
                print(f"Updated {pf}: {old_version} → {version}")
            except Exception as e:
                print(f"Warning: Could not update {pf}: {e}")

    def parse_version(self, version_str):
        """Parse version string into major.minor.patch"""
        parts = version_str.split('.')
        if len(parts) != 3:
            raise ValueError(f"Invalid version format: {version_str}. Expected major.minor.patch")

        try:
            return [int(part) for part in parts]
        except ValueError:
            raise ValueError(f"Invalid version format: {version_str}. All parts must be integers")

    def format_version(self, major, minor, patch):
        """Format version components into string"""
        return f"{major}.{minor}.{patch}"

    def bump_patch(self):
        """Increment patch version"""
        current = self.read_version()
        major, minor, patch = self.parse_version(current)
        new_version = self.format_version(major, minor, patch + 1)
        self._update_version(new_version)
        return new_version

    def bump_minor(self):
        """Increment minor version, reset patch to 0"""
        current = self.read_version()
        major, minor, patch = self.parse_version(current)
        new_version = self.format_version(major, minor + 1, 0)
        self._update_version(new_version)
        return new_version

    def bump_major(self):
        """Increment major version, reset minor and patch to 0"""
        current = self.read_version()
        major, minor, patch = self.parse_version(current)
        new_version = self.format_version(major + 1, 0, 0)
        self._update_version(new_version)
        return new_version

    def set_version(self, version_str):
        """Set specific version"""
        # Validate format
        self.parse_version(version_str)
        self._update_version(version_str)
        return version_str

    def _update_version(self, new_version, update_files=None):
        """Update version in all configured files"""
        # Default to config settings
        if update_files is None:
            update_files = self.config.get('files', {})
        
        # Always update VERSION file
        self.write_version(new_version)
        
        # Update other files based on config or flags
        if update_files.get('setup.py', True):
            self.update_setup_py(new_version)
        
        if update_files.get('package.json', False):
            self.update_package_json(new_version)

    def create_config(self):
        """Create a .versionrc config file"""
        # Detect what files exist
        has_setup = self.setup_file.exists()
        package_files = self.find_package_json_files()
        
        config = {
            "files": {
                "version": True,
                "setup.py": has_setup,
                "package.json": len(package_files) > 0
            },
            "package_json_paths": [str(pf) for pf in package_files],
            "auto_discover": True
        }
        
        with open('.versionrc', 'w') as f:
            json.dump(config, f, indent=2)
            f.write('\n')
        
        print("Created .versionrc with detected settings")
        return config

    @staticmethod
    def get_version(version_file="VERSION", default_version="0.1.0"):
        """
        Get version from file, create with default if missing.

        Args:
            version_file: Path to version file (default: "VERSION")
            default_version: Default version if file doesn't exist (default: "0.1.0")

        Returns:
            Version string
        """
        vm = VersionManager(version_file)

        try:
            return vm.read_version()
        except FileNotFoundError:
            # Create VERSION file with default version
            vm.write_version(default_version)
            return default_version


__all__ = ['VersionManager']